/* Example: pointers and two-dimensional arrays */

#include <stdio.h>

main()
{
  int arr[2][3] = {{11, 22, 33},
                   {44, 55, 66}};  /* an array of ints */
  int *ptr;                        /* a pointer to int */
  int i;                           /* row index */
  int j;                           /* column index */
  
  ptr = &arr[0][0];  /* set pointer ptr to point at the start of the 
                     array, i.e. ptr contains the address of arr[0][0], 
                     so arr[0][0] is the "pointee" of ptr. 

  We could also write ptr = *arr, because this means &(**arr). Now, 
  **arr means *(*arr), which means *(arr[0]), which means arr[0][0].
 
  We could also write ptr = arr[0]; because ptr is a pointer
  to int and so is arr[0]: it points to arr[0][0]. 
   
  We could also write ptr = *arr; because ptr is a pointer
  to int (type int *) and arr is a pointer to a pointer to int 
  (type int **). */
   
  /* We now have several ways of accessing the array elements. */
  
  /* As a two-dimensional array: */
  for (i = 0; i < 2; i++)
  {
    for (j = 0; j < 3; j++)
      printf("%i ", arr[i][j]);  /* the most natural */
    putchar('\n');
  }
  putchar('\n');
  
  /* As two one-dimensional arrays: */
  for (j = 0; j < 3; j++)
    printf("%i ", *(arr[0] + j));
  putchar('\n');
  
  for (j = 0; j < 3; j++)
    printf("%i ", *(arr[1] + j));
  putchar('\n');
  putchar('\n');
  
  /* As a single one-dimensional array: */
  for (j = 0; j < 6; j++)
    printf("%i ", *(*arr + j));
  putchar('\n');
  putchar('\n');

  /* Or, equivalently: */
  for (j = 0; j < 6; j++)
    printf("%i ", *(arr[0] + j));
  putchar('\n');
  putchar('\n');
  
  /* Suppose we know only the number of elements (6) and have a pointer 
  to the first element (ptr). We have to make an assumption about the 
  array shape. */
  
  /* Assuming a one-dimensional array (1 row by 6 columns): */
  for (j = 0; j < 6; j++)
    printf("%i ", ptr[j]);
  putchar('\n');
  putchar('\n');
  
  /* Assuming a 2 x 3 array (2 rows by 3 columns): */
  for (i = 0; i < 2; i++)
  {
    for (j = 0; j < 3; j++)
      printf("%i ", *(ptr + 3*i + j));
    putchar('\n');
  }
  putchar('\n');

  /* Assuming a 3 x 2 array (3 rows by 2 columns): */
  for (i = 0; i < 3; i++)
  {
    for (j = 0; j < 2; j++)
      printf("%i ", *(ptr + 2*i + j));
    putchar('\n');
  }
  putchar('\n');

  /* Assuming a 6 x 1 array (6 rows by 1 column): */
  for (i = 0; i < 6; i++)
  {
    printf("%i ", *(ptr + i));
    putchar('\n');
  }
  putchar('\n');
}
